#!perl

# Perl script to check disk partition alignments under Windows Vista or later.
# Just run it, with no parameters.
#
# If you don't have a recent version of Perl installed, you can use the standalone
# Win32 Perl 4 here (extract PERL4W32.EXE and rename it to Perl4.exe):
# http://www.burtonsys.com/download/dbf2csv.zip
#
# Refs:
#   Align SSD: http://www.acronis.com/en-gb/support/documentation/ATIH2014/21807.html
#   TRIM SSD: http://www.acronis.com/en-gb/support/documentation/ATIH2014/15920.html
#
# For WMIC Help, from the Windows command prompt type:
#   wmic /?
#   wmic partition /?
#   wmic partition get /?
#
#
# By Dave Burton, www.geeksalive.com
# Uncopyrighted -- do whatever you wish with it!


# immediate output of debug prints
$| = 1;


# Run the "wmic" command to get the starting byte offset of each partition:
if (!open(PARTITIONS,"wmic partition get BlockSize, Index, Name, Size, StartingOffset|")) {
   # Note: Depending on what version of Perl you're using(?), if wmic.exe
   # is not found, the attemt to run it might emit the message to stderr:
   #   'wmic' is not recognized as an internal or external command,
   #   operable program or batch file.
   # If the command fails, abort our script:
   die "ERROR: could not run 'wmic', $!\n(It is normally in C:\\Windows\\System32\\Wbem\\wmic.EXE)\n";
}
# With some version of Perl, if it fails it appears to succeed, but is already at EOF:
if (eof(PARTITIONS)) {
   die "ERROR: could not run 'wmic'\n(It is normally in C:\\Windows\\System32\\Wbem\\wmic.EXE)\n";
}

# Here's an example of the output of that wmic command
# (it happens to be a 480GB SSD, a 1.5 TB HDD, and an 8 GB thumbdrive)
#
# BlockSize  Index  Name                   Size           StartingOffset
# 512        0      Disk #0, Partition #0  469361491968   1048576
# 512        1      Disk #0, Partition #1  10740563968    469362540544
# 512        0      Disk #1, Partition #0  1478280347648  1048576
# 512        1      Disk #1, Partition #1  22020096000    1478281396224
# 512        0      Disk #3, Partition #0  7896203264     65536
#
# Note that the output is actually in Unicode, but Perl is apparently smart
# enough to magically convert it.


while (<PARTITIONS>) {  # for each line of wmic output...
   chop;
   $_ =~ s/[\r\n]*$//;  # because Perl 4 lacks chomp (to strip both cr and lf)
   $ln = $_;

   # trim one more blank off the end
   $last2chars = substr($ln, length($ln)-3, 2);
   if ('  ' eq $last2chars) {
      $ln = substr($ln,0,length($ln)-1);
   }

   # split the line into fields
   @tmp = split( /\s+/, $ln );
   # $n = $#tmp + 1;
   # print "dbg: $n fields\n";

   # shorten the lines a bit more (so output is usually under 80 chars/line)
   $ln =~ s/Name                   Size/Name             Size/;  #shorten header line
   $ln =~ s/Partition \#/Ptn \#/;  #shorten data line

   # print each line, followed by the alignment assessment
   print "$ln";
   if (7 == $#tmp) {
      $startingoffset = $tmp[7];
      if ($startingoffset =~ /^\d+$/) {
         # there are 8 fields, and last one is numeric
         # $n = 0 + $startingoffset;
         # print "dbg: $n = startingoffset\n";
         if (($startingoffset % (1024*1024)) == 0) {
            print " (1MB aligned)";
         } elsif (($startingoffset % (1024*512)) == 0) {
            print " (512KB aligned, but not 1MB aligned)";
         } elsif (($startingoffset % 4096) == 0) {
            print " (4KB aligned, but not 512KB aligned)";
         } elsif (($startingoffset % 512) == 0) {
            print " *** UNALIGNED! ***";
         } elsif (($startingoffset % 512) == 0) {
            print " *** ERROR! StartingOffset not divisble by 512! ***";
         }
      }
   }
   print "\n";
}

# Here's a sample of what the output might look like:
#
# BlockSize  Index  Name             Size           StartingOffset
# 512        0      Disk #0, Ptn #0  469361491968   1048576         (1MB aligned)
# 512        1      Disk #0, Ptn #1  10740563968    469362540544    (1MB aligned)
# 512        0      Disk #1, Ptn #0  1478280347648  1048576         (1MB aligned)
# 512        1      Disk #1, Ptn #1  22020096000    1478281396224   (1MB aligned)
# 512        0      Disk #3, Ptn #0  7896203264     65536           (4KB aligned, but not 512KB aligned)

